#!/bin/sh
### ------------------------------------------------------------------
### | Copyright Unpublished, Atria Software, Inc.  All Rights        |
### | Reserved.  This software contains proprietary and confidential |
### | information of Atria and its suppliers.  Use, disclosure or    |
### | reproduction is prohibited without the prior express written   |
### | consent of Atria.                                              |
### ------------------------------------------------------------------
###
#
# Extract versions from a ClearCase delta container file.
#
# The filenames of the extracted version's cleartext are of the form:
#     <elem_id>.<branch_id>.<hex_version_number>
#
# If possible, the VOB will be consulted to provide useful names:
#     <elem_id> = the leafname of the file element (e.g., "foo.c")
#     <branch_id> = the branch type name of the version (e.g., "mybranch")
#
# When the VOB doesn't contain know about the element or the branch 
# instance oids the following values are used:
#     <elem_id> = the element's object id (oid)
#     <branch_id> = a container relative hex branch instance number
#
# A element branch map file is also created (branch_map.<elem_id>).  
# This file is useful for determining the element's version tree structure.
# The map contains information of the form:
#  branch <hex_branch_num> (<branch_id>) off of branch <hex_branch_num> version <hex_version_num>
# E.g.,
#  # Branch map for element bar.c
#  branch 4 (9ceb9cdb.07f711d0.a4fc.00:01:80:7a:0e:c9) off of branch 3 version 2
#  branch 3 (/main/mybranch) off of branch 0 version 4
#  branch 0 (/main) off of branch 0 version 0
#

ATRIA_DIR=${ATRIAHOME:-/usr/atria}
PROGN=`basename $0`
ECHO=/bin/echo
UNAME=/bin/uname
SLEEP=/bin/sleep
SED=/bin/sed
GREP=/bin/grep
OS_TYPE=`$UNAME -s`
OS_VER=`$UNAME -r`
VERBOSE="FALSE"
if [ "$1" = "-v" ] ; then
    VERBOSE="TRUE"
fi
DATE_TO_TIME=$ATRIA_DIR/etc/utils/date2time
TIME_TO_DATE=$ATRIA_DIR/etc/utils/time2date

if [ "$CLEARCASE_ROOT" = "" ] ; then
    $ECHO "Warning: You have no view set so pathnames won't be printed."
else
    if $ATRIA_DIR/bin/cleartool describe .@@ >/dev/null 2>/dev/null ; then
	echo >/dev/null
    else
       $ECHO "Warning: You are not in a VOB so pathnames won't be printed."
    fi
fi

sgi_echo_n() { $ECHO "$*\c" ; }
hp_echo_n() { $ECHO "$*\c" ; }
osf1_echo_n() { $ECHO "$*\c" ; }
sun4_echo_n() { $ECHO -n "$*" ; }
sun5_echo_n() { $ECHO "$*\c" ; }
aix_echo_n() { $ECHO -n "$*" ; }
unix_sv_echo_n() { $ECHO -n "$*" ; }

case "$OS_TYPE" in

IRIX*)
	ECHO_N=sgi_echo_n
	;;

'SunOS')
	case "$OS_VER" in
	  4.*)
	    ECHO_N=sun4_echo_n
	    ;;
	  *)
	    ECHO_N=sun5_echo_n
	    ;;
	esac
	;;

'HP-UX')
	ECHO_N=hp_echo_n
	;;

'OSF1')
	ECHO_N=osf1_echo_n
	;;

'AIX')
	ECHO_N=aix_echo_n
	;;

'UNIX_SV')
	ECHO_N=unix_sv_echo_n
	;;

*)
	$ECHO "*** $PROGN *** Unknown OS type: $OS_TYPE"
	exit -1
	;;
esac

SINCE_TIME=""

$ECHO_N "Where is the VOB storage directory? "
read VOB_STORAGE_ROOT_PN
if [ ! -r $VOB_STORAGE_ROOT_PN/s ] ; then
    $ECHO "$VOB_STORAGE_ROOT_PN doesn't look like a vob storage root!"
    exit 1
fi
VOB_TAG=`$ATRIA_DIR/bin/cleartool lsvob -l -storage $VOB_STORAGE_ROOT_PN | $GREP '^Tag:' | $SED 's/Tag: //'`
if [ "$VOB_TAG" = "" ] ; then
    $ECHO "Warning: Unable to determine VOB tag."
else
    $ECHO "Using VOB tag: $VOB_TAG"
fi

$ECHO ""
while : ; do
    $ECHO_N "Scan ONLY the pool lost+found areas <y,n>? "
    read ONLY_LAF
    case $ONLY_LAF in
      y) break ;;
      n) break ;;
      *) $ECHO "illegal response \"$ONLY_LAF\"" ;;
    esac
done
if [ "$ONLY_LAF" = "y" ] ; then
    for i in $VOB_STORAGE_ROOT_PN/s/*/lost+found ; do
	$ECHO "Will scan \"$i\""
	SCAN_DIRS="$SCAN_DIRS $i"
    done
    if [ "$SCAN_DIRS" = "" ] ; then
	$ECHO "There are no lost+found areas to scan"
	exit 1
    fi
else
    SCAN_DIRS="$VOB_STORAGE_ROOT_PN/s"
    $ECHO "Will scan all of \"$SCAN_DIRS\""
fi

$ECHO ""
$ECHO_N "Recover versions since when (ccase-date-str or <CR> for LATEST)? "
read SINCE_TIME
if [ "$SINCE_TIME" != "" ] ; then
    SINCE_TIME=`$DATE_TO_TIME $SINCE_TIME`
    if [ "$SINCE_TIME" = "" ] ; then
	exit 1
    fi
    $ECHO "Will recover versions since \"`$TIME_TO_DATE $SINCE_TIME`\""
fi

if [ "$SINCE_TIME" = "" ] ; then
    VERBOSE="TRUE"
fi

$ECHO ""
$ECHO_N "Recover only versions before when (ccase-date-str or <CR> all versions)? "
read BEFORE_TIME
if [ "$BEFORE_TIME" != "" ] ; then
    BEFORE_TIME=`$DATE_TO_TIME $BEFORE_TIME`
    if [ "$BEFORE_TIME" = "" ] ; then
	exit 1
    fi
    $ECHO "Will recover versions before \"`$TIME_TO_DATE $BEFORE_TIME`\""
fi

$ECHO ""
$ECHO_N "Recover all branches (<CR> for all, '0' for just /main)? "
read ONLY_BRANCH

$ECHO ""
$ECHO_N "Where do you want the results (<CR> for "/usr/tmp")? "
read TO_DIR_PN

if [ "$TO_DIR_PN" = "" ] ; then
    TO_DIR_PN="/usr/tmp"
fi

/bin/find $SCAN_DIRS -type f -print | while read CNTR_PN ; do

    TMP_CNTR=`basename $CNTR_PN | $GREP 'tmp_*'`
    if [ "$TMP_CNTR" != "" ] ; then
	continue # skip unstable temp containers
    fi

    ELEM_OID=`$GREP '^\^E [0-9a-f:.]*$' $CNTR_PN | $SED -e 's/\^E //'`
    if [ "$ELEM_OID" = "" ] ; then
	# Announce non-tfd containers so at least folks are informed
	# of additional things available in the pool.
	$ECHO
	$ECHO "#### not a text file delta container: $CNTR_PN"
	continue
    fi

    if [ "$VERBOSE" = "TRUE" -a "$VOB_TAG" != "" ] ; then
	ELEM_PN="`$ATRIA_DIR/bin/cleartool dump oid:$ELEM_OID@$VOB_TAG | $GREP '\@\@'`"
	ELEM_LEAF="`basename $ELEM_PN @@`"
    else
	ELEM_PN="$ELEM_OID"
	ELEM_LEAF="$ELEM_OID"
    fi

    if [ "$ELEM_PN" = "" ] ; then
	# Couldn't find element -- may be new
	ELEM_PN="$ELEM_OID"
	ELEM_LEAF="$ELEM_OID"
    fi

    if [ "$VERBOSE" = "TRUE" ] ; then
	$ECHO "Considering $ELEM_PN ..."
    else
	$ECHO_N "."
    fi

    RECOVERED_MARKER="$TO_DIR_PN/recovered_vers.$$"
    rm -f $RECOVERED_MARKER

    $GREP '^\^V [0-9a-f:.]*' $CNTR_PN | \
	while read CTRL VER_OID BRANCH GEN DTM ; do
	    # Do only a specific branch?
	    if [ "$ONLY_BRANCH" != "" ] ; then
		if [ "$ONLY_BRANCH" != "$BRANCH" ] ; then
		    continue; # not an interesting one
		fi
	    fi

	    # Do since a date?
	    if [ "$SINCE_TIME" != "" ] ; then
  	        # Do those later than given date
	        if [ "`($ECHO $SINCE_TIME; $ECHO $DTM) | sort | head -1`" = "$DTM" ] ; then
		    break # all done, get out of loop
	        fi
	    fi

	    # Do before a date?
	    if [ "$BEFORE_TIME" != "" ] ; then
  	        # Do those before given date
	        if [ "`($ECHO $BEFORE_TIME; $ECHO $DTM) | sort | head -1`" = "$BEFORE_TIME" ] ; then
		    continue # all done, get out of loop
	        fi
	    fi

	    if [ "$BRANCH" = "0" ] ; then
		BRANCH="main"
	    fi

	    # DO IT!
	    # Tell user what's happening
	    if [ "$VERBOSE" = "FALSE" ] ; then
		# stop printing ....... and get the element pathname
		$ECHO
		if [ "$VOB_TAG" != "" ] ; then
		    ELEM_PN="`$ATRIA_DIR/bin/cleartool dump oid:$ELEM_OID@$VOB_TAG | $GREP '\@\@'`"
		    ELEM_LEAF="`basename $ELEM_PN @@`"
		else
		    ELEM_PN=""
		fi
		if [ "$ELEM_PN" = "" ] ; then
		    # Couldn't find element -- may be new
		    ELEM_PN="$ELEM_OID"
		    ELEM_LEAF="$ELEM_OID"
		fi
	    fi
	    $ECHO "  Fetching $ELEM_PN to $TO_DIR_PN/$ELEM_LEAF.$BRANCH.$GEN"
	    $ECHO "    [from delta container $CNTR_PN]"

	    # FETCH VERSION
	    # Command line for constructing (fetching) a version
	    # <manager pathname>     source_container_pname
	    #  	    	    	 copy_into_pname
	    #  	    	    	 ver_oid
	    #
	    $ATRIA_DIR/lib/mgrs/text_file_delta/construct_version \
		$CNTR_PN "$TO_DIR_PN/$ELEM_LEAF.$BRANCH.$GEN" $VER_OID

	    touch $RECOVERED_MARKER

	    # Only do latest?
	    if [ "$SINCE_TIME" = "" ] ; then
	        break
	    fi
        done # versions
    
    # if any versions were extracted, provide a branch num -> name map
    if [ -r $RECOVERED_MARKER ] ; then
	rm -f $RECOVERED_MARKER
	if [ "$VOB_TAG" != "" ] ; then
	    ELEM_PN="`$ATRIA_DIR/bin/cleartool dump oid:$ELEM_OID@$VOB_TAG | $GREP '\@\@'`"
	    ELEM_LEAF="`basename $ELEM_PN @@`"
        else
	    ELEM_PN=""
	fi
	if [ "$ELEM_PN" = "" ] ; then
	    ELEM_PN=$ELEM_OID
	    ELEM_LEAF=$ELEM_OID
	fi
	BRANCH_MAP_FILE="$TO_DIR_PN/branch_map.$ELEM_LEAF"
	# Note: only append to map file. That can result in extra swill
	# but in the case where there are multiple containers for a file
	# we want to be sure we capture all of the data possible
	$ECHO "Branch map for $ELEM_LEAF in $BRANCH_MAP_FILE"
	$ECHO "# Branch map for element $ELEM_LEAF" >>$BRANCH_MAP_FILE

	$GREP '^\^B [0-9a-f:.]*' $CNTR_PN | \
	    while read CTRL BR_OID BRANCH OFF_BRANCH OFF_VER ; do

            if [ "$BRANCH" = "0" ] ; then
                BR_PNAME="main"
            else
                if [ "$VOB_TAG" != "" ] ; then
                    BR_PNAME="`$ATRIA_DIR/bin/cleartool dump oid:$BR_OID | $GREP '\@\@' | $SED -e 's/.*@@//'`"
                else
                    BR_PNAME=""
                fi
                if [ "$BR_PNAME" = "" ] ; then
                    BR_PNAME="$BR_OID"
                fi
            fi
            $ECHO "branch $BRANCH ($BR_PNAME) off of branch $OFF_BRANCH version $OFF_VER" >>$BRANCH_MAP_FILE

            done # branches
        $ECHO "====="
    fi
    
done # find
